#!/usr/bin/perl
# NOTES:
# The MRTDATADIR environment variable MUST BE SET before running this script

# To Do:  Validate, stitch and mosaic the Alaska images

use Getopt::Long;
#use Date::Calc;
use Time::Local;

my $verbose = 0;

# fdir - Location of the MODIS data stage directory
# Change the verbosity level for more information about the processing
# 0 - Nothing
# 1 - More information
# 2 - Most information
GetOptions('fdir=s' => \$fdir,'v=s' => \$verbose);

#determine the current date
my($sec, $min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
$curryear = $year + 1900;
my $startyear = 2007;
my %adatelist;
my $product = "MOD13A1";
my $version = "005";
my $productsubset = "*";
# Turn mosaic'ing on and off by setting boolmosaic
my $boolmosaic = 1;

# Separate Aqua and Terra MODIS data

@afiles = `ls $fdir/MYD*`;
print qq{Looking in directory: $fdir\n};
for ($yr=$startyear;$yr <= $curryear;$yr++)
{
	
	ClearHash();
	@files = `ls $fdir/$product\.A$yr*\.$productsubset*`;
	
	if(@files) 
	{
		
		foreach $tile(@files)
		{
			chomp($tile);
			next unless $tile =~ /$product\.A$yr(\d{3})\.$productsubset/;
			AssignPair($1,1);
			if($verbose > 1) {print qq{Terra Tiles for dates: $yr $1 $tile\n}; }
	
		}

		
		my $numkeys = keys %adatelist;
		if($verbose) { print qq{Found $numkeys distinct dates\n};}
	
		foreach $key (keys %adatelist)
		{
	
			if($verbose) 
			{ 
				print("$yr $key $adatelist{$key}\n");
			}
			
			
			#`ls $fdir/$product\.A$yr$key\.$productsubset*.hdf > $fdir/modis_filelist_$yr$key\.txt`;
			# Determine whether or not all the files for the US exist for this particular date	
			if(ValidateCompleteUS("$yr$key"))
			{
				print "Yearday: $yr$key is a complete dataset\n";
				# If all the files exist, create the CONUS mosaic
				if($boolmosaic) 
				{
					print "Starting stitch and mosaic process for $yr$key\n";
					MosaicMODIS("$fdir/us_modis_file_list_$product$yr$key\.txt",$fdir,"$yr$key"); 
				}
			
			}		
							
		}
		
	}

}

# End of the main program loop 
sub AssignPair
{
	
	my($key,$value) = @_;
	
	if (defined($adatelist{$key}))
	{
		#print qq{already defined, incrementing $key...\n};
		$adatelist{$key}++;
	}
	else
	{
		#print qq{not yet defined, adding $key...\n};
		$adatelist{$key} = 1;
	}

}

sub ClearHash
{
	#deletes all the elements from the adatelist hash

	while(($key,$value) = each %adatelist)
	{
	
		delete ($adatelist{$key});
	}
	


}

# ValidateCompleteUS reads a list of MODIS tiles and determines if all the data
# exist to mosaic the tiles for the continental United States

sub ValidateCompleteUS
{

	$fulldate = shift;
	my $i = 1;
	my $success = 0;
	
	my $usmodistilelist = "/data/scripts/modis_tile_list_us.txt";
	my $akmodistilelist = "/data/scripts/modis_tile_list_ak.txt";
	

	open (TILELIST, $usmodistilelist);
   	while ($tileid = <TILELIST>) 
	{
		chomp $tileid;
		$testfile = "$fdir/$product\.A$fulldate\.$tileid\.$version\.\*\.hdf";
		@files = `ls $testfile`;
				
		# Check to see if the file exists.  If it does, create or 
		# add to the file list.
		
		if ($files[0])
		{
			if($verbose) {print "File exists: $files[0]\n";}
			if($i == 1)
			{
				`ls $testfile > $fdir/us_modis_file_list_$product$fulldate.txt`;
			}
			else
			{
				`ls $testfile >> $fdir/us_modis_file_list_$product$fulldate.txt`;
			}
		}
		else
		{
			return 0;
		}
	
      		$i++;
   	}

   	close(TILELIST);
	return 1;


}

######################
# Mosaic and Resample the images
######################


sub MosaicMODIS
{

	$filelist = shift;
	$filedir = shift;
	$date = shift;
	$regionname = "us";
	$prj = "/data/scripts/lazea.prm";
	$outtiff = "$regionname\_$date\.tif";
	print qq{Creating TIFF file: $outtiff\n};
	print qq{Mosaicing $filelist using directory $filedir for day $date\n};
	# Run the Mosaic Process
	print "Starting MRT Mosaic process\n";
	my $result = `/usr/local/bin/mrtmosaic -i $filelist -o $filedir/$regionname\_$date\_composite.hdf -s ["1 1 0 0 0 0 0 0 0 0 0"]`;
	print qq{$result\n};
	print "Completed MRT Mosaic process\n";
	
	# Run the Reproject Process
	print "Starting Reprojection\n";
	if($regionname eq "us")
    	{
    		$subset = "-2050500 752500 2536500 -2136500";
	}
	
    	$result = `/usr/local/bin/resample -p $prj -i $filedir/$regionname\_$date\_composite.hdf -o $filedir/$outtiff -a OUTPUT_PROJ_COORDS -l "$subset" `;

	print qq{$result\n};
    	print "Completed MRT Resample process\n";
    	## Clean up the unneeded files
    	print "Removing Composite HDF File\n";
    	$result = `rm $filedir/$regionname\_$date\_composite.hdf`;
    	print qq{$result\n};
    	print "Removed Composite HDF File\n";
        
}
